<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use FiloBlu\Refilo\Remote\Entity\EntityProviderReaderHandlerInderface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObject;

/**
 * Class BaseProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
abstract class BaseProvider implements EntityProviderInterface
{
    /** @var array */
    protected $items = [];

    /** @var array|null */
    protected $ids;

    /**
     * @var EntityProviderReaderHandlerInderface|null
     */
    protected $readerHandlder;
    /**
     * @var DataObject
     */
    protected $readerHandelerArguments;

    /**
     * @var int
     */
    protected $pageSize = 500;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * AbstractFromCollectionProvider constructor.
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig
    )
    {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @return mixed|void
     */
    public function current()
    {
        return current($this->items);
    }

    /**
     * @return mixed|void
     */
    public function next()
    {
        return next($this->items);
    }

    /**
     * @return bool|float|int|string|null
     */
    public function key()
    {
        return key($this->items);
    }

    /**
     * @return bool|mixed
     */
    public function valid()
    {
        return current($this->items);
    }

    /**
     * @param EntityProviderReaderHandlerInderface $handler
     * @param DataObject|null $arguments
     * @return EntityProviderInterface
     */
    public function attachReaderHandler(EntityProviderReaderHandlerInderface $handler, DataObject $arguments = null): EntityProviderInterface
    {
        $this->readerHandlder = $handler;
        $this->readerHandelerArguments = $arguments ?: new DataObject();
        return $this;
    }

    /**
     * @return EntityProviderReaderHandlerInderface
     */
    public function detachReaderHanlder(): EntityProviderReaderHandlerInderface
    {
        $handler = $this->readerHandlder;
        $this->readerHandlder = null;
        return $handler;
    }

    /**
     * @return EntityProviderReaderHandlerInderface|null
     */
    public function getReadHandler()
    {
        return $this->readerHandlder;
    }

    /**
     * @return bool
     */
    public function hasReadHandler(): bool
    {
        return $this->readerHandlder !== null;
    }

    /**
     * @return DataObject
     */
    public function getReadHandlerArguments()
    {
        return $this->readerHandelerArguments;
    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @return int
     */
    public function getBulkSize(): int
    {
        $pageSize = (int)$this->scopeConfig->getValue('sync/data/index_bulk_size');
        $this->pageSize = ($pageSize) ?: $this->pageSize;
        return (int)$this->pageSize;
    }
}
